# coding: utf-8
"""
Copyright 2016 Assetic

"""
from __future__ import absolute_import

import os
import shutil
import time
import sys
import base64
import openpyxl #to read excel files
import io
import array

try:
    import configparser as configparser
except:
    #V2 python?
    import ConfigParser as configparser

#import assetic libraries we'll use
from assetic.api_client import ApiClient
from assetic.configuration import Configuration
from assetic.apis import DataExchangeJobApi
from assetic.apis import DataExchangeTaskApi
from assetic.apis import DocumentApi
from assetic.models import Assetic3IntegrationRepresentationsFilePropertiesRepresentation
from assetic.models import Assetic3IntegrationRepresentationsDocumentRepresentation
from assetic.models import Assetic3WebApiModelsDataExchangeDataExchangeJobVM
from assetic.rest import ApiException

class BulkProcesses(object):
    """
    Class to manage bulk upload
    """

    def __init__(self, configfile = None):
        if configfile == None:
            configfile = os.path.join(os.getcwd(), 'assetic.ini')

        if os.path.isfile(configfile) == False:
            message = "Configuration file not found.  Expecting assetic.ini in current folder, or set via module init"
            print(message)
            raise ValueError(message)
        else:
            self.configfile = configfile

    def bulkdocument_upload(self, csvfile, myDataSharedDirectoryFilePath, debug=False):
        """
            Script to automate document/photo upload in bulk fashion (Based on Assetic.BulkDocumentUpload.py)
            Usage:
            1. Invoke the script: python.exe -c "import assetic; uploadapi = assetic.BulkProcesses('C:/Users/xyz/assetic.ini'); uploadapi.bulkdocument_upload('C:/Users/xyz/data.csv', 'C:/Users/xyz)"

            Requirements:
            1. Pass a csv/xlsx/xls file containing columns as follows in EXACT order
            Complex Asset Id | Document/Photos file name | Is Key-photo

            Output:
            Processed document/photos upload request status is written back to passed/specified CSV/XLSX/XLS file

            Parameters -
            csvfile: CSV/XLS/XLSX file contaning document/photos data to import
            myDataSharedDirectoryFilePath: Root directory location where myData Shared directory contents are stored (i.e. Directory Location from myData configuration file)
            debug (optiona;): values True or False.  Default is False
        """

        # Read ini
        settings = configparser.ConfigParser()
        configfile = self.configfile
        settings.read(configfile)
        # Apply config settings
        conf = Configuration()
        conf.host = settings.get('environment', 'url')
        conf.username = settings.get('auth', 'username')
        conf.password = settings.get('auth', 'api_key')
        auth = conf.get_basic_auth_token()

        #Debug settings
        #conf.logger_file = os.path.join(os.path.dirname(csvfile), 'BulkDocumentPhotosUploadLog.txt')
        conf.debug = debug

        client = ApiClient(conf.host,"Authorization",auth)
        docapi = DocumentApi(client)
        dataexchangeapi = DataExchangeJobApi(client)

        print ('File for bulk document upload operation is specified: {0}\n'.format(csvfile))
        validDocumentToImportExtensions = {".csv", ".xlsx", ".xls"}
        if (os.path.exists(csvfile) and any(csvfile.endswith(ext) for ext in validDocumentToImportExtensions)):
            try:
                wb = openpyxl.load_workbook(csvfile)
                sheet = wb.active
                sheet['D' + str(1)] = 'Migrated Document Id'
                sheet['E' + str(1)] = 'Operation Performed On'
                sheet['F' + str(1)] = 'Status'
                wb.save(csvfile)
            except Exception as e:
                print('{0}\n'.format(e))
            else:
                for row in range(2, sheet.max_row + 1):
                    complexAssetId  = sheet['A' + str(row)].value
                    fileNameToUpload   = sheet['B' + str(row)].value
                    isKeyPhoto = sheet['C' + str(row)].value

                    print ('Processing {2}/{3}: Complex Asset Id: [{0}]; File name: [{1}]'.format(complexAssetId, fileNameToUpload, row, sheet.max_row))

                    fileToUploadPath = ''
                    isDocument = False;
                    myDataSharedDirectoryDocumentsFilePath = os.path.join(myDataSharedDirectoryFilePath, 'Documents')
                    for path, subdirs, files in os.walk(myDataSharedDirectoryDocumentsFilePath):
                        for fileItem in files:
                            fullPath = os.path.join(path, fileItem)
                            fetchCorrectFile = True if fileItem == fileNameToUpload else False
                            if (fetchCorrectFile):
                                fileToUploadPath = fullPath
                                isDocument = True
                                break;


                    if fileToUploadPath == '':
                        myDataSharedDirectoryPhotosFilePath = os.path.join(myDataSharedDirectoryFilePath, 'Photos')
                        for path, subdirs, files in os.walk(myDataSharedDirectoryPhotosFilePath):
                            for fileItem in files:
                                fullPath = os.path.join(path, fileItem)
                                fetchCorrectFile = True if fileItem == fileNameToUpload else False
                                if (fetchCorrectFile):
                                    fileToUploadPath = fullPath
                                    break;


                    if (fileToUploadPath != '' and os.path.exists(fileToUploadPath)):
                        fileToUploadPath = os.path.abspath(fileToUploadPath)
                        fileToUploadName  = os.path.basename(fileToUploadPath)
                        #fileExtention = fileToUploadPath[fileToUploadPath.rindex('.') + 1:]
                        fileExtention = os.path.splitext(fileToUploadPath)[1][1:]
                        with open(fileToUploadPath, "rb") as f:
                            filename = f.name
                            data = f.read()

                            if sys.version_info < (3,0):
                                filecontents = data.encode("base64")
                            else:
                                filecontents = base64.b64encode(data)
                                filecontents = filecontents.decode(encoding="utf-8", errors="strict")

                            filesize = os.path.getsize(filename)

                            file_properties = Assetic3IntegrationRepresentationsFilePropertiesRepresentation()
                            file_properties.name = fileToUploadName
                            file_properties.file_size = filesize
                            file_properties.mimetype = fileExtention
                            file_properties.filecontent = filecontents
                            filearray = [file_properties]

                            validImageExtensions = {".jpg", ".png", ".gif"}
                            isFileAnImage = any(fileToUploadName.endswith(ext) for ext in validImageExtensions)

                            if isDocument:
                                documentGroupId = 10
                            else:
                                documentGroupId = 7

                            try:
                                document = Assetic3IntegrationRepresentationsDocumentRepresentation()
                                document.is_key_photo = True if isFileAnImage == True and isKeyPhoto == True and isDocument == False else False
                                document.document_group = documentGroupId
                                document.parent_type = 'ComplexAsset'
                                document.parent_id = complexAssetId
                                document.label = 'Importing complex asset document/photo - [' + fileToUploadName + '] on ' + time.strftime("%d/%m/%Y %H:%M:%S")
                                document.file_property = filearray
                                document.document_size = filesize
                                document.mime_type = fileExtention

                                doc = docapi.document_post(document)
                            except ApiException as e:
                                print('Error occured while uploading document/photo: [{0}]; Error status {1}, Reason: {2}\n'.format(fileToUploadName, e.status,e.reason))
                            else:
                                docid = doc[0].get('Id')
                                if docid != '':
                                    print ('UPLOADED document/photo: [{1}]; Response: [{0}]\n'.format(docid, fileToUploadName))                                                                       
                                else:
                                    print ('Upload FAILED for [{0}]\n'.format(fileToUploadName))

                                isDocumentUploadSuccessful = True if docid != '' else False
                                if isDocumentUploadSuccessful:
                                    sheet['D' + str(row)] = docid
                                    sheet['E' + str(row)] = time.strftime("%c")
                                    sheet['F' + str(row)] = 'SUCCESS'
                                else:
                                    sheet['D' + str(row)] = 'N/A'
                                    sheet['E' + str(row)] = time.strftime("%c")
                                    sheet['F' + str(row)] = 'Error; Please check integration application logs for details'

                                wb.save(csvfile)
                    else:
                        print ('File: [{0}] is non-existing\n'.format(fileToUploadPath))
        else:
            print ('File: [{0}] is not specified for bulk document/photos upload operation or not of correct format\n'.format(csvfile))

    def bulkkeyphotos_upload(self, keyPhotosStorageLocation, debug=False):
        """
            Script to automate key-photos upload in bulk fashion (Based on Assetic.BulkKeyPhotosUpload.py)
            Usage:
            1. Change your directory to where all key-photos are stored
            2. Invoke the script: python.exe -c "import assetic; uploadapi = assetic.BulkProcesses('C:/Users/xyz/assetic.ini'); uploadapi.bulkkeyphotos_upload()"

            Output:
            Keyphotos upload request status is written back to file: KeyPhotosMigrationStatus.xlsx in directory defined in Step: 1

            Parameters -
            keyPhotosStorageLocation: File path where key-photos are stored
            debug (optiona;): values True or False.  Default is False
        """

        # Read ini
        settings = configparser.ConfigParser()
        configfile = self.configfile
        settings.read(configfile)
        # Apply config settings
        conf = Configuration()
        conf.host = settings.get('environment', 'url')
        conf.username = settings.get('auth', 'username')
        conf.password = settings.get('auth', 'api_key')
        auth = conf.get_basic_auth_token()

        #Debug settings
        #conf.logger_file = os.path.join(os.path.dirname(keyPhotosStorageLocation), 'BulkKeyPhotosUploadLog.txt')
        conf.debug = debug

        client = ApiClient(conf.host,"Authorization",auth)
        docapi = DocumentApi(client)
        dataexchangeapi = DataExchangeJobApi(client)

        print ('Key-photo storage location specified: {0}'.format(keyPhotosStorageLocation))
        ##Keyphoto file naming format: <SaaS AssetID>_keyphoto.jpg
        excelSaveFilePath = os.path.join(os.path.dirname(keyPhotosStorageLocation), 'KeyPhotosMigrationStatus.xlsx')

        keyPhotosFiles = os.listdir(keyPhotosStorageLocation)
        #noOfFiles  = len(os.walk(keyPhotosStorageLocation).next()[2])
        noOfFiles = len(next(os.walk(keyPhotosStorageLocation))[2])

        validImageExtensions = {".jpg", ".png", ".gif"}

        try:
            wb = openpyxl.Workbook()
            sheet = wb.active
            sheet.title = "Key-photo Migration status"
            sheet['A' + str(1)] = 'Operation Performed On'
            sheet['B' + str(1)] = 'File Name'
            sheet['C' + str(1)] = 'Migrated Document Id'
            sheet['D' + str(1)] = 'Status'

            wb.save(excelSaveFilePath)
        except Exception as e:
            print('{0}\n'.format(e))
        else:
            noOfKeyPhotosToUpload = len([keyPhotoFileItem for keyPhotoFileItem in keyPhotosFiles if(any(keyPhotoFileItem.endswith(ext) for ext in validImageExtensions))])
            itemCounter = 0

            for keyPhotoFile in keyPhotosFiles:
                fullPath = os.path.join(keyPhotosStorageLocation, keyPhotoFile)
                splitedFileName, fileExtention = os.path.splitext(keyPhotoFile) #[1][1:]
                fileExtention = fileExtention[1:]
                assetId = keyPhotoFile.split('_')[0]

                if(any(keyPhotoFile.endswith(ext) for ext in validImageExtensions)):
                    itemCounter = itemCounter + 1
                    if (os.path.exists(fullPath)):
                        with open(fullPath, "rb") as f:
                            filename = os.path.basename(f.name)
                            data = f.read()

                        if sys.version_info < (3,0):
                            filecontents = data.encode("base64")
                        else:
                            filecontents = base64.b64encode(data)
                            filecontents = filecontents.decode(encoding="utf-8", errors="strict")

                        filesize = os.path.getsize(fullPath)

                        file_properties = Assetic3IntegrationRepresentationsFilePropertiesRepresentation()
                        file_properties.name = filename
                        file_properties.file_size = filesize
                        file_properties.mimetype = fileExtention
                        file_properties.filecontent = filecontents
                        filearray = [file_properties]

                        try:
                            document = Assetic3IntegrationRepresentationsDocumentRepresentation()
                            document.is_key_photo = True
                            document.document_group = 7 #Photos items Id from DOCCBGroup table
                            document.parent_type = 'ComplexAsset'
                            document.parent_id = assetId
                            document.label = 'Importing complex asset key-photo - [' + filename + '] on ' + time.strftime("%d/%m/%Y %H:%M:%S")
                            document.file_property = filearray
                            document.document_size = filesize
                            document.mime_type = fileExtention

                            doc = docapi.document_post(document)
                        except ApiException as e:
                            print('Error occured while uploading key-photos: [{0}]; Error status {1}, Reason: {2}\n'.format(filename, e.status,e.reason))
                        else:                            
                            docid = doc[0].get('Id')
                            if docid != '':                                
                                print ('[{3}/{4}]: Uploaded file: [{1}] associated with assetId: [{2}]; Response: [{0}]\n'.format(docid, filename, assetId, itemCounter, noOfKeyPhotosToUpload))
                            else:    
                                print ('[{2}/{3}]: UPLOAD FAILED for [{0}] associated with assetId: [{1}]\n'.format(filename, assetId, itemCounter, noOfKeyPhotosToUpload)) 

                            isDocumentUploadSuccessful = True if docid != '' else False
                            for row in range(sheet.max_row + 1, sheet.max_row + 2):
                                sheet['A' + str(row)] = time.strftime("%c")
                                sheet['B' + str(row)] = keyPhotoFile
                                if isDocumentUploadSuccessful:
                                    sheet['C' + str(row)] = docid
                                    sheet['D' + str(row)] = 'SUCCESS'
                                else:
                                    sheet['C' + str(row)] = 'N/A'
                                    sheet['D' + str(row)] = 'Error; Please check integration application logs for details'

                                wb.save(excelSaveFilePath)
                    else:
                        print ('File: [{0}] is non-existing\n'.format(keyPhotoFile))
                else:
                    print ('File: [{0}] is not valid image for key-photos upload operation and will not be processed\n'.format(keyPhotoFile))

    def dataexchange_upload(self, csvfile, debug=False):
        """
            Script to automate data-exchange in bulk fashion (Assetic.BulkDataExchange.py)
            Usage:
            1. Invoke the script: python.exe -c "import assetic; uploadapi = assetic.BulkProcesses('C:/Users/xyz/assetic.ini'); uploadapi.dataexchange_upload('C:/Users/xyz/data.csv')"

            Requirements:
            1. Pass a csv/xlsx/xls file containing columns as follows in EXACT order
            ProfileId | Document to Import (Complete file-path)

            Output:
            Processed data-exchange request status is written back to passed/specified CSV/XLSX/XLS file

            Parameters -
            csvfile: CSV/XLS/XLSX file contaning data-exchange profile and respecitive document to import pairs
            debug (optiona;): values True or False.  Default is False
        """

        # Read ini
        settings = configparser.ConfigParser()
        configfile = self.configfile
        settings.read(configfile)
        # Apply config settings
        conf = Configuration()
        conf.host = settings.get('environment', 'url')
        conf.username = settings.get('auth', 'username')
        conf.password = settings.get('auth', 'api_key')
        auth = conf.get_basic_auth_token()

        #Debug settings
        #conf.logger_file = os.path.join(os.path.dirname(csvfile), 'BulkDataExchangeUploadLog.txt')
        conf.debug = debug

        client = ApiClient(conf.host,"Authorization",auth)
        docapi = DocumentApi(client)
        dataexchangeapi = DataExchangeJobApi(client)

        validDocumentToImportExtensions = {".csv", ".xlsx", ".xls"}
        if (os.path.exists(csvfile) and any(csvfile.endswith(ext) for ext in validDocumentToImportExtensions)):
            try:
                wb = openpyxl.load_workbook(csvfile)
                sheet = wb.active
                sheet['C' + str(1)] = 'Imported Document Id'
                sheet['D' + str(1)] = 'Data-exchange Job Id'
                sheet['E' + str(1)] = 'Operation Performed On'
                sheet['F' + str(1)] = 'Status'
                wb.save(csvfile)
            except Exception as e:
                print('{0}\n'.format(e))
            else:
                totalItems = sheet.max_row - 1
                for row in range(2, sheet.max_row + 1):
                    itemCounter = row - 1                    
                    dataExchangeProfileId = sheet['A' + str(row)].value
                    documentFilePathToImport = sheet['B' + str(row)].value

                    filename, fileExtention = os.path.splitext(documentFilePathToImport) #[1][1:]
                    fileExtention = fileExtention[1:]

                    print ('Processing {2}/{3}: Profile Id: [{0}]; Document file name: [{1}]; Document to import: [{4}]'.format(dataExchangeProfileId, filename, row, sheet.max_row, documentFilePathToImport))
                    if (os.path.exists(documentFilePathToImport)):
                        with open(documentFilePathToImport, "rb") as f:
                            data = f.read()

                        if sys.version_info < (3,0):
                            filecontents = data.encode("base64")
                        else:
                            filecontents = base64.b64encode(data)
                            filecontents = filecontents.decode(encoding="utf-8", errors="strict")

                        filesize = os.path.getsize(documentFilePathToImport)

                        file_properties = Assetic3IntegrationRepresentationsFilePropertiesRepresentation()
                        file_properties.name = filename
                        file_properties.file_size = filesize
                        file_properties.mimetype = fileExtention
                        file_properties.filecontent = filecontents
                        filearray = [file_properties]

                        try:
                            document = Assetic3IntegrationRepresentationsDocumentRepresentation()
                            document.label = 'Importing data-exchange document - [' + filename + '] on ' + time.strftime("%d/%m/%Y %H:%M:%S")
                            document.file_property = filearray
                            document.document_size = filesize
                            document.mime_type = fileExtention
                            
                            doc = docapi.document_post(document)
                        except ApiException as e:
                            print ('Error occured while uploading data-exchange document: [{0}]; Error status {1}, Reason: {2}\n'.format(filename, e.status,e.reason))
                        else:
                            docid = doc[0].get('Id')                            
                            if docid != '':
                                print ('[{3}/{4}]: UPLOADED Document: [{0}] associated with data-exchange profile: [{1}]; Document response: [{2}]\n'.format(filename, dataExchangeProfileId, docid, itemCounter, totalItems))
                            else:
                                print ('[{2}/{3}]: Document upload FAILED for [{1}]; data-exchange profile: {0}\n'.format(dataExchangeProfileId, filename, itemCounter, totalItems))
                                
                            isDocumentUploadSuccessful = True if docid != '' else False
                            if isDocumentUploadSuccessful:
                                try:
                                    job = Assetic3WebApiModelsDataExchangeDataExchangeJobVM()
                                    job.profile_id = dataExchangeProfileId
                                    job.document_id = docid
                                    task = dataexchangeapi.data_exchange_job_post(job)
                                except ApiException as e:
                                    sheet['C' + str(row)] = docid
                                    sheet['D' + str(row)] = 'N/A'
                                    sheet['E' + str(row)] = time.strftime("%c")
                                    sheet['F' + str(row)] = 'Error; ' + str(e.status) + ': ' + e.reason
                                    print('Error occured while requesting data-exchange task for profile: [{0}]; Error status {1}, Reason: {2}\n'.format(dataExchangeProfileId, e.status,e.reason))
                                else:
                                    print('Requested Data-exchange task: [{0}] for profile: [{1}] and document: [{2}]\n'.format(task, dataExchangeProfileId, filename))

                                    sheet['C' + str(row)] = docid
                                    sheet['D' + str(row)] = task
                                    sheet['E' + str(row)] = time.strftime("%c")
                                    sheet['F' + str(row)] = 'SUCCESS'
                            else:
                                sheet['C' + str(row)] = 'N/A'
                                sheet['D' + str(row)] = 'N/A'
                                sheet['E' + str(row)] = time.strftime("%c")
                                sheet['F' + str(row)] = 'Error; Document upload failed; Please check integration application logs for details'

                            wb.save(csvfile)
                    else:
                        print ('[{2}/{3}]: Document to import: [{0}] is non-existing for data-exchange profile: [{1}]\n'.format(filename, dataExchangeProfileId, itemCounter, totalItems))
        else:
            print ('File: [{0}] is not specified for bulk data-exchange operation or not of correct format\n'.format(csvfile))

    def dataexchange_get_error_summary(self, csvfile, errorFilesStorageDirectory, debug=False):
        """
            Script to fetch data-exchange task summary in bulk fashion
            Usage:
            1. Invoke the script: python.exe -c "import assetic; uploadapi = assetic.BulkProcesses('C:/Users/xyz/assetic.ini'); uploadapi.dataexchange_get_error_summary('C:/Users/xyz/data.csv', 'C:/Users/xyz/')"

            Requirements:
            1. Pass a csv/xlsx/xls file used for containing columns as follows in EXACT order
            ProfileId | Document to Import (Complete file-path) | Imported Document Id | Data-exchange Job Id | Operation Performed On | Status

            Output:
            Fetches error document and summary of failed data-exchange tasks and status is written back to passed/specified CSV/XLSX/XLS file with error document in specified error directory

            Parameters -
            csvfile: CSV/XLS/XLSX file contaning data-exchange profile, imported document id and requested data-exchange task id
            errorFilesStorageDirectory: Directorty location where error document are to be stored for reference
            debug (optiona;): values True or False.  Default is False
        """

        # Read ini
        settings = configparser.ConfigParser()
        configfile = self.configfile
        settings.read(configfile)
        # Apply config settings
        conf = Configuration()
        conf.host = settings.get('environment', 'url')
        conf.username = settings.get('auth', 'username')
        conf.password = settings.get('auth', 'api_key')
        auth = conf.get_basic_auth_token()

        #Debug settings
        #conf.logger_file = os.path.join(os.path.dirname(csvfile), 'BulkDataExchangeErrorSummaryLog.txt')
        conf.debug = debug

        client = ApiClient(conf.host,"Authorization",auth)
        docapi = DocumentApi(client)
        dataExchangeTaskapi = DataExchangeTaskApi(client)

        validDocumentToImportExtensions = {".csv", ".xlsx", ".xls"}
        if (os.path.exists(csvfile) and any(csvfile.endswith(ext) for ext in validDocumentToImportExtensions)):
            try:
                wb = openpyxl.load_workbook(csvfile)
                sheet = wb.active
                sheet['G' + str(1)] = 'Error Document Id'
                sheet['H' + str(1)] = 'Summary'
                wb.save(csvfile)
            except Exception as e:
                print('{0}\n'.format(e))
            else:
                for row in range(2, sheet.max_row + 1):
                    importedDocumentId = sheet['C' + str(row)].value
                    dataExchangeTaskId = sheet['D' + str(row)].value

                    if dataExchangeTaskId != None and dataExchangeTaskId != 'N/A':
                        print ('Processing {1}/{2}: Fetching document: [{3}] for data-exchange Task Id: [{0}]'.format(dataExchangeTaskId, row, sheet.max_row, importedDocumentId))
                        try:
                            task = dataExchangeTaskapi.data_exchange_task_get(dataExchangeTaskId)
                        except ApiException as e:
                            sheet['G' + str(row)] = e.status
                            sheet['H' + str(row)] = e.reason
                            print('Error occured while fetching status of data-exchange task:[{0}]; Error status {1}, Reason: {2}\n'.format(dataExchangeTaskId, e.status, e.reason))
                        else:
                            status = task.get('Status')
                            statusDescription = task.get('StatusDescription')
                            if status == int('4'):
                                errorDocumentId = task.get('ErrorDocumentId')
                                if errorDocumentId != None:
                                    print('Fetching error documnent: [{0}] for data-exchange task: [{2}] into [{1}]'.format(errorDocumentId, errorFilesStorageDirectory, dataExchangeTaskId))
                                    self.document_get(errorFilesStorageDirectory, errorDocumentId)
                                    sheet['G' + str(row)] = errorDocumentId
                                    sheet['H' + str(row)] = 'Status: ' + str(status) + '/' + statusDescription + ' : ' + task.get('Summary')
                                else:
                                    print('Data-exchange task: [{0}] is in status: [{1}] and have response summary: [{2}] but NO ERROR DOCUMENT is recorded\n'.format(dataExchangeTaskId, status, statusDescription))
                                    sheet['G' + str(row)] = 'N/A'
                                    sheet['H' + str(row)] = 'Status: ' + str(status) + '/' + statusDescription + ' : ' + task.get('Summary')
                            else:
                                print('Data-exchange task: [{0}] is in status: [{1}/{2}]\n'.format(dataExchangeTaskId, status, statusDescription))
                                sheet['G' + str(row)] = 'N/A'
                                sheet['H' + str(row)] = 'Status: ' + str(status) + '/' + statusDescription + ' : ' + task.get('Summary')

                    wb.save(csvfile)
        else:
            print ('File: [{0}] is not specified for to operate bulk data-exchange fetch summary operation or not of correct format\n'.format(csvfile))

    def document_get(self, fileStorageDirectoryLocation, docid, debug=False):
        """
            Script to fetch document
            Usage:
            1. Invoke the script: python.exe -c "import assetic; uploadapi = assetic.BulkProcesses('C:/Users/xyz/assetic.ini'); uploadapi.document_get('C:/Users/xyz/', '1234-34566-23456-2334324')"

            Requirements:
            1. Pass directory location where error document can be stored
            2. Document Id of document to fetch

            Output:
            Saves the document in directory location specified by [fileStorageDirectoryLocation]

            Parameters -            
            errorFilesStorageDirectory: Directorty location where error document are to be stored for reference
            docid: Document Id of document to be fetched
            debug (optiona;): values True or False.  Default is False
        """
        
        # Read ini
        settings = configparser.ConfigParser()
        configfile = self.configfile
        settings.read(configfile)
        # Apply config settings
        conf = Configuration()
        conf.host = settings.get('environment', 'url')
        conf.username = settings.get('auth', 'username')
        conf.password = settings.get('auth', 'api_key')
        auth = conf.get_basic_auth_token()

        #Debug settings
        #conf.logger_file = os.path.join(os.path.dirname(fileStorageDirectoryLocation), 'FetchDocumentLog.txt')
        conf.debug = debug
        
        client = ApiClient(conf.host,"Authorization", auth , None, 'Content-Disposition')
        docapi = DocumentApi(client)

        if (os.path.exists(fileStorageDirectoryLocation) and docid != None and docid != ''):
            try:
                getfile = docapi.document_get_document_file(docid)
            except ApiException as e:
                print('Error occured while fetching Document:[{0}]; Error status {1}, Reason: {2}\n'.format(docid, e.status, e.reason))
            else:
                getfile = docapi.document_get_document_file(docid)
                if getfile != None:
                    if 'attachment' in getfile[1] and 'filename=' in getfile[1]:
                        filename = getfile[1].split('filename=',1)[1]
                        if '"' in filename or "'" in filename:
                            filename = filename[1:-1]
                        fullfilename = os.path.join(fileStorageDirectoryLocation, filename)
                    else:
                        fullfilename = os.path.join(fileStorageDirectoryLocation, docid)

                    #if file with name already exists then add document id at the end to make it unique
                    if os.path.exists(fullfilename):
                        fileNameToModify, fileExtention = os.path.splitext(fullfilename)                        
                        fullfilename = fileNameToModify + '_' + docid + '.' + fileExtention[1:]
                    
                    data = getfile[0]
                    if sys.version_info >= (3,0):
                        try:
                            data = data.decode('utf-8')
                        except:
                            data = data #pass

                    with open(fullfilename, 'wb' ) as out_file:
                        out_file.write(data)
                        print ('Created file: [{0}]\n'.format(fullfilename))
                        
##                    if type(data) == bytes:
##                        with open( fullfilename, 'wb' ) as out_file:
##                            out_file.write(getfile[0])
##                            print ("Created bytes file: {0}/{1}".format(fullfilename, getfile[1]))
##                    elif type(data) == str:
##                        with open( fullfilename, 'w', newline='' ) as out_file:
##                            out_file.write(getfile[0])
##                            print ("Created string file: {0}/{1}".format(fullfilename, getfile[1]))
##                    else:
##                        print("File not created, data type unhandled: {0}".format(str(type(data))))
                        
                else:
                    print('No document fetch response is received')
        else:
            print('Either File storage directory location or document id is not specified or is in incorrect format')

